]> andersk Git - openssh.git/blobdiff - ssh.c
- markus@cvs.openbsd.org 2001/04/05 21:05:24
[openssh.git] / ssh.c
diff --git a/ssh.c b/ssh.c
index c76c7bb71f530e7c48c3f2146e867b0162927c50..278e7eda4a494fc4fb7752bcc878610d915a5bdf 100644 (file)
--- a/ssh.c
+++ b/ssh.c
@@ -39,7 +39,7 @@
  */
 
 #include "includes.h"
-RCSID("$OpenBSD: ssh.c,v 1.100 2001/03/01 22:46:37 markus Exp $");
+RCSID("$OpenBSD: ssh.c,v 1.106 2001/04/05 21:05:24 markus Exp $");
 
 #include <openssl/evp.h>
 #include <openssl/err.h>
@@ -130,11 +130,8 @@ struct sockaddr_storage hostaddr;
  */
 volatile int received_window_change_signal = 0;
 
-/* Flag indicating whether we have a valid host private key loaded. */
-int host_private_key_loaded = 0;
-
 /* Host private key. */
-RSA *host_private_key = NULL;
+Key *host_private_key = NULL;
 
 /* Original real UID. */
 uid_t original_real_uid;
@@ -175,6 +172,7 @@ usage(void)
 
        fprintf(stderr, "  -c cipher   Select encryption algorithm: "
            "``3des'', ``blowfish''\n");
+       fprintf(stderr, "  -m macs     Specify MAC algorithms for protocol version 2.\n");
        fprintf(stderr, "  -p port     Connect to this port.  Server must be on the same port.\n");
        fprintf(stderr, "  -L listen-port:host:port   Forward local port to remote address\n");
        fprintf(stderr, "  -R listen-port:host:port   Forward remote port to local address\n");
@@ -232,7 +230,7 @@ rsh_connect(char *host, char *user, Buffer * command)
 
 int    ssh_session(void);
 int    ssh_session2(void);
-int    guess_identity_file_type(const char *filename);
+void   load_public_identity_files(void);
 
 /*
  * Main program for the ssh client.
@@ -387,7 +385,7 @@ main(int ac, char **av)
                                options.log_level++;
                                break;
                        } else {
-                               fatal("Too high debugging level.\n");
+                               fatal("Too high debugging level.");
                        }
                        /* fallthrough */
                case 'V':
@@ -543,22 +541,24 @@ main(int ac, char **av)
        /* Do not allocate a tty if stdin is not a tty. */
        if (!isatty(fileno(stdin)) && !force_tty_flag) {
                if (tty_flag)
-                       log("Pseudo-terminal will not be allocated because stdin is not a terminal.\n");
+                       log("Pseudo-terminal will not be allocated because stdin is not a terminal.");
                tty_flag = 0;
        }
 
        /* Get user data. */
        pw = getpwuid(original_real_uid);
        if (!pw) {
-               log("You don't exist, go away!\n");
+               log("You don't exist, go away!");
                exit(1);
        }
        /* Take a copy of the returned structure. */
        pw = pwcopy(pw);
 
-       /* Initialize "log" output.  Since we are the client all output
-          actually goes to the terminal. */
-       log_init(av[0], options.log_level, SYSLOG_FACILITY_USER, 0);
+       /*
+        * Initialize "log" output.  Since we are the client all output
+        * actually goes to stderr.
+        */
+       log_init(av[0], SYSLOG_LEVEL_INFO, SYSLOG_FACILITY_USER, 1);
 
        /* Read per-user configuration file. */
        snprintf(buf, sizeof buf, "%.100s/%.100s", pw->pw_dir, _PATH_SSH_USER_CONFFILE);
@@ -571,7 +571,9 @@ main(int ac, char **av)
        fill_default_options(&options);
 
        /* reinit */
-       log_init(av[0], options.log_level, SYSLOG_FACILITY_USER, 0);
+       log_init(av[0], options.log_level, SYSLOG_FACILITY_USER, 1);
+
+       seed_rng();
 
        if (options.user == NULL)
                options.user = xstrdup(pw->pw_name);
@@ -626,12 +628,8 @@ main(int ac, char **av)
         * privileges, because the file is only readable by root.
         */
        if (ok && (options.protocol & SSH_PROTO_1)) {
-               Key k;
-               host_private_key = RSA_new();
-               k.type = KEY_RSA1;
-               k.rsa = host_private_key;
-               if (load_private_key(_PATH_HOST_KEY_FILE, "", &k, NULL))
-                       host_private_key_loaded = 1;
+               host_private_key = key_load_private_type(KEY_RSA1,
+                   _PATH_HOST_KEY_FILE, "", NULL);
        }
        /*
         * Get rid of any extra privileges that we may have.  We will no
@@ -675,15 +673,11 @@ main(int ac, char **av)
                }
                exit(1);
        }
-       /* Expand ~ in options.identity_files, known host file names. */
-       /* XXX mem-leaks */
-       for (i = 0; i < options.num_identity_files; i++) {
-               options.identity_files[i] =
-                   tilde_expand_filename(options.identity_files[i], original_real_uid);
-               options.identity_files_type[i] = guess_identity_file_type(options.identity_files[i]);
-               debug("identity file %s type %d", options.identity_files[i],
-                   options.identity_files_type[i]);
-       }
+       /* load options.identity_files */
+       load_public_identity_files();
+
+       /* Expand ~ in known host file names. */
+       /* XXX mem-leaks: */
        options.system_hostfile =
            tilde_expand_filename(options.system_hostfile, original_real_uid);
        options.user_hostfile =
@@ -694,12 +688,12 @@ main(int ac, char **av)
            tilde_expand_filename(options.user_hostfile2, original_real_uid);
 
        /* Log into the remote system.  This never returns if the login fails. */
-       ssh_login(host_private_key_loaded, host_private_key,
-                 host, (struct sockaddr *)&hostaddr, original_real_uid);
+       ssh_login(host_private_key, host, (struct sockaddr *)&hostaddr,
+           original_real_uid);
 
        /* We no longer need the host private key.  Clear it now. */
-       if (host_private_key_loaded)
-               RSA_free(host_private_key);     /* Destroys contents safely */
+       if (host_private_key != NULL)
+               key_free(host_private_key);     /* Destroys contents safely */
 
        exit_status = compat20 ? ssh_session2() : ssh_session();
        packet_close();
@@ -952,9 +946,6 @@ ssh_session2_callback(int id, void *arg)
 
        debug("client_init id %d arg %ld", id, (long)arg);
 
-       if (no_shell_flag)
-               goto done;
-
        if (tty_flag) {
                struct winsize ws;
                char *cp;
@@ -1017,15 +1008,14 @@ ssh_session2_callback(int id, void *arg)
        }
        /* channel_callback(id, SSH2_MSG_OPEN_CONFIGMATION, client_init, 0); */
 
-done:
        /* register different callback, etc. XXX */
        packet_set_interactive(interactive);
 }
 
 int
-ssh_session2(void)
+ssh_session2_command(void)
 {
-       int window, packetmax, id;
+       int id, window, packetmax;
        int in, out, err;
 
        if (stdin_null_flag) {
@@ -1047,14 +1037,6 @@ ssh_session2(void)
        if (!isatty(err))
                set_nonblock(err);
 
-       /* XXX should be pre-session */
-       ssh_init_forwarding();
-
-       /* If requested, let ssh continue in the background. */
-       if (fork_after_authentication_flag)
-               if (daemon(1, 1) < 0)
-                       fatal("daemon() failed: %.200s", strerror(errno));
-
        window = CHAN_SES_WINDOW_DEFAULT;
        packetmax = CHAN_SES_PACKET_DEFAULT;
        if (!tty_flag) {
@@ -1066,29 +1048,48 @@ ssh_session2(void)
            window, packetmax, CHAN_EXTENDED_WRITE,
            xstrdup("client-session"), /*nonblock*/0);
 
+debug("channel_new: %d", id);
+
        channel_open(id);
        channel_register_callback(id, SSH2_MSG_CHANNEL_OPEN_CONFIRMATION,
             ssh_session2_callback, (void *)0);
 
-       return client_loop(tty_flag, tty_flag ? options.escape_char : -1, id);
+       return id;
 }
 
 int
-guess_identity_file_type(const char *filename)
+ssh_session2(void)
 {
-       struct stat st;
+       int id;
+
+       /* XXX should be pre-session */
+       ssh_init_forwarding();
+
+       id = no_shell_flag ? -1 : ssh_session2_command();
+
+       /* If requested, let ssh continue in the background. */
+       if (fork_after_authentication_flag)
+               if (daemon(1, 1) < 0)
+                       fatal("daemon() failed: %.200s", strerror(errno));
+
+       return client_loop(tty_flag, tty_flag ? options.escape_char : -1, id);
+}
+
+void
+load_public_identity_files(void)
+{
+       char *filename;
        Key *public;
-       int type = KEY_RSA1; /* default */
+       int i;
 
-       if (stat(filename, &st) < 0) {
-               /* ignore this key */
-               return KEY_UNSPEC;
-       }
-       public = key_new(type);
-       if (!load_public_key(filename, public, NULL)) {
-               /* ok, so we will assume this is 'some' key */
-               type = KEY_UNSPEC;
+       for (i = 0; i < options.num_identity_files; i++) {
+               filename = tilde_expand_filename(options.identity_files[i],
+                   original_real_uid);
+               public = key_load_public(filename, NULL);
+               debug("identity file %s type %d", filename,
+                   public ? public->type : -1);
+               xfree(options.identity_files[i]);
+               options.identity_files[i] = filename;
+               options.identity_keys[i] = public;
        }
-       key_free(public);
-       return type;
 }
This page took 0.069164 seconds and 4 git commands to generate.